@@ -1,5 +1,5 @@
 /*
-* RTP/RTCP packet dumper (November 1995 RTP draft).
+* RTP/RTCP packet dumper (RFC 1889)
 * Usage:
 *   rtpdump [address/port] > dump file
 *
@@ -23,7 +23,7 @@
 *  with suitably chosen file names. Each such file will be
 *  individually playable. (Gaps?)
 *
-* Copyright (c) 1995-1996 by H. Schulzrinne, GMD Fokus
+* Copyright (c) 1995-1997 by H. Schulzrinne (GMD Fokus, Columbia U.)
 * $Id$
 */
 
@@ -404,13 +404,13 @@ static int parse_control(char *buf, int len)
           (int)r->r.rr.ssrc, r->common.p, r->common.count, r->common.length);
         for (i = 0; i < r->common.count; i++) {
           printf("(ssrc=%0lx fraction=%g lost=%lu last_seq=%lu jit=%lu lsr=%lu dlsr=%lu)\n",
-           ntohl(r->r.sr.rr[i].ssrc),
-           r->r.sr.rr[i].fraction / 256.,
-           ntohl(r->r.sr.rr[i].lost),
-           ntohl(r->r.sr.rr[i].last_seq),
-           ntohl(r->r.sr.rr[i].jitter),
-           ntohl(r->r.sr.rr[i].lsr),
-           ntohl(r->r.sr.rr[i].dlsr));
+           ntohl(r->r.rr.rr[i].ssrc),
+           r->r.rr.rr[i].fraction / 256.,
+           ntohl(r->r.rr.rr[i].lost),
+           ntohl(r->r.rr.rr[i].last_seq),
+           ntohl(r->r.rr.rr[i].jitter),
+           ntohl(r->r.rr.rr[i].lsr),
+           ntohl(r->r.rr.rr[i].dlsr));
         }
         printf(" )\n"); 
         break;
