@@ -33,12 +33,15 @@
 *    multicast VAT <- translator <- unicast VAT
 *
 *
-* this means that the audio agent on the unicast link should be able to use
+* This means that the audio agent on the unicast link should be able to use
 * both VAT and RTP.
 *
 * Author: Dorgham Sisalem
 * GMD Fokus, Berlin
 *
+* Bug fixes and supstantial improvements by 
+* Stephen Casner <casner@precept.com>
+* 
 */
 
 #include <sys/types.h>
@@ -51,11 +54,10 @@
 #include <stdio.h>
 #include <signal.h>
 #include <errno.h>
-#if !defined(nextstep)
 #include <values.h>
-#endif
 #include <unistd.h>      /* select(), perror() */
 #include <stdlib.h>      /* getopt(), atoi() */
+#include <memory.h>      /* memset() */
 #include "rtp.h"
 #include "rtpdump.h"
 #include "ansi.h"
@@ -64,18 +66,17 @@
 #include "vat.h"
 
 #define PAD(x,n) (((n) - ((x) & (n-1))) & (n-1))
-
-static char rcsid[] = "$Id: rtptrans.c,v 1.1 1995/08/17 15:13:17 hgs Exp $";
-
 #define MAX_HOST 10
 
+extern int gettimeofday();
+
 static int debug = 0;
 static int hostc = 0;
-static multi_sock[2];
+static multi_sock[3] = {0,0,0};
 static struct {
   int sock;
   struct sockaddr_in sin;
-} side[MAX_HOST][2];  /* [host][proto] */
+} side[MAX_HOST][3];  /* [host][proto] */
 
 /* 
  * create a linked list for traversing the sequence numbers of the different 
@@ -84,25 +85,28 @@ static struct {
 typedef struct stream_id{
   int seq;
   int addr;
+  int next_ts;
   struct stream_id* next;
 } STREAM;
 
 typedef struct stream_id stream;
-/* we need to keep in memory the sequence nimber of the last sent packet
- * for each data stream
+
+/* 
+ * We need to keep in memory the sequence number of the last sent packet
+ * for each data stream.
  */
-static stream *head;  /* start of list */
-static stream *middle;/* middle of the list, keeping this pointer reduces the
-		       * avarage searching path to a 1/4 of the list instead of
-		       * 1/2.
-		       */
-static stream *last;  /* last seen stream, as the probability that the next
-		       * packet will belong to the last seen one this reduces
-		       * the searching path even further
-		       */
+static stream *head;   /* start of list */
+static stream *middle; /* middle of the list, keeping this pointer reduces the
+                        * avarage searching path to a 1/4 of the list instead of
+                        * 1/2.
+                        */
+static stream *last;   /* Last seen stream, as the probability that the next
+                        * packet will belong to the last seen one this reduces
+                        * the searching path even further.
+                        */
 static int list_len;
 
-int create_stream(int addr)
+int create_stream(int addr, int next)
 {
   stream *elem;
   int i;
@@ -115,6 +119,7 @@ int create_stream(int addr)
   new_stream->next=NULL;
   new_stream->addr=addr;
   new_stream->seq=addr;
+  new_stream->next_ts=next;
   if(head==NULL) { /* initialize the list */
     head=new_stream;
     middle=new_stream;
@@ -122,7 +127,7 @@ int create_stream(int addr)
 #if !defined(nextstep)
     srand48(rand());  /* initialize random number generator */
 #else
-	srand(rand());	/* (fred) This is surprising */
+    srand(rand());    /* (fred) This is surprising */
 #endif
     new_stream->seq=rand();
     last=new_stream;
@@ -158,16 +163,21 @@ int create_stream(int addr)
   return new_stream->seq;
 }
 
-/* 
- * add a stream to the list
- */
 
-int find_stream(int addr)
+/* 
+ * Add a stream to the list.
+ */
+int find_stream(int addr, int ts, int next, int m)
 {
   stream *element;
-  if((last!=NULL)&&(addr==last->addr)) /* packet belongs to the last seen 
-					  stream */
+  /* packet belongs to the last seen stream */
+  if ((last!=NULL)&&(addr==last->addr)) {
+     last->seq+=1;
+     if (ts != last->next_ts && !m)
+       last->seq+=1;   /* approximate missing some packets */
+     last->next_ts = next;
      return(last->seq);
+  }
   if((middle!=NULL)&&(addr>=middle->addr))
     element=middle;
   else
@@ -175,10 +185,13 @@ int find_stream(int addr)
   for(; (element!=NULL)&&(element->addr<=addr); element=element->next) {
     if(element->addr==addr) {
       element->seq+=1;
+       if (ts != element->next_ts && !m)
+         element->seq += 1;  /* approximate missing some packets */
+       element->next_ts = next;
       return (element->seq);
     }
   }
-  return(create_stream(addr));
+  return(create_stream(addr, next));
 }
 
 struct sdes_msg {
@@ -206,16 +219,17 @@ static Notify_value socket_handler(Notify_client client, int sock)
   addr_len = sizeof(sin_from);
   len = recvfrom(sock, packet, sizeof(packet), 0,
         (struct sockaddr *)&sin_from, &addr_len);
+  rtp_hdr=(rtp_hdr_t *)packet;
   if (debug) {
     struct timeval now;
 
     gettimeofday(&now, 0);
     printf("%0.3f %s %4d [%s/%d]\n", 
       now.tv_sec + now.tv_usec/1e6, 
-      proto ? "RTCP" : "RTP ", len,
+      rtp_hdr->version==2 ? (proto ? "RTCP" : "RTP ") :
+        rtp_hdr->version==0 ? (proto ? "vatC" : "vat ") : "UKWN", len,
       inet_ntoa(sin_from.sin_addr), ntohs(sin_from.sin_port));
   }
-  rtp_hdr=(rtp_hdr_t *)packet;
 
   /* do not translate packets that already use RTP or arrive over the unicast
    link*/
@@ -223,45 +237,56 @@ static Notify_value socket_handler(Notify_client client, int sock)
     rtp_hdr=(rtp_hdr_t *)packet;   
     for (i = 0; i < hostc; i++) {
       if (side[i][proto].sock != sock) {
-	if (sendto(side[i][proto].sock, packet, len, 0,
-	  (struct sockaddr *)&side[i][proto].sin,sizeof(side[i][proto].sin))<0)
-	  perror("sendto RTCP");
+        if (sendto(side[i][2].sock, packet, len, 0,
+          (struct sockaddr *)&side[i][proto].sin,sizeof(side[i][proto].sin))<0)
+        perror("sendto RTCP");
       }
     }
   }
   else {
     struct msghdr msg;
-    if(!proto) { /*translate VAT packets */
+    if (!proto) { /*translate VAT packets */
       struct iovec iov[2];
       char type; 
+      int samples = len-VAT_LEN;
       vat_hdr=(vat_hdr_t *)packet;   
+
       if(vat_hdr->flags&VATHF_NEWTS)
-	rtp_hdr_send.m=1;
+        rtp_hdr_send.m=1;
       else
-	rtp_hdr_send.m=0;
+        rtp_hdr_send.m=0;
       type= vat_hdr->flags&VATHF_FMTMASK;
+
       switch (type) {
-      case VAT_AUDF_MULAW8:
-      case VAT_AUDF_G721:
+
       case VAT_AUDF_GSM:
-      case VAT_AUDF_G723 :
-	rtp_hdr_send.pt=type;
-	break;
+        samples = (samples/33)*160;
+
+      case VAT_AUDF_MULAW8:
+      case VAT_AUDF_G721:   /* samples not right for this */
+      case VAT_AUDF_G723:   /* samples not right for this */
+        rtp_hdr_send.pt=type;
+        break;
+
       case VAT_AUDF_IDVI:
-	rtp_hdr_send.pt=5;
-	break;
+        samples = (samples-4)*2;
+        rtp_hdr_send.pt=5;
+        break;
+
       case VAT_AUDF_L16_16:
       case VAT_AUDF_L16_44:
       case VAT_AUDF_LPC4:
       case VAT_AUDF_CELP:
       case VAT_AUDF_LPC1:
       case VAT_AUDF_UNDEF :
-      default:
-	perror(" unknown codecs ");
-	break;
+        default:
+        rtp_hdr_send.pt=115;   /* hopefully unused */
+        perror(" unknown codecs ");
+        break;
       }
       rtp_hdr_send.ssrc    = sin_from.sin_addr.s_addr;
-      rtp_hdr_send.seq     = find_stream(rtp_hdr_send.ssrc);
+      rtp_hdr_send.seq     = find_stream(rtp_hdr_send.ssrc, vat_hdr->ts,
+         vat_hdr->ts + samples, rtp_hdr_send.m);
       rtp_hdr_send.version = RTP_VERSION;
       rtp_hdr_send.p       = 0;
       rtp_hdr_send.x       = 0;
@@ -275,32 +300,42 @@ static Notify_value socket_handler(Notify_client client, int sock)
       msg.msg_iov = iov;
       msg.msg_iovlen = 2;
       for (i = 0; i < hostc; i++) {
-	if (side[i][proto].sock != sock) {
-	  msg.msg_name = (caddr_t ) &side[i][proto].sin;
-	  msg.msg_namelen = sizeof(side[i][proto].sin);
-	  msg.msg_accrights = 0;
-	  msg.msg_accrightslen = 0;
-	  if ((sendmsg(side[i][proto].sock, &msg,0))!= 
-	    iov[0].iov_len +iov[1].iov_len)
-	    perror("sendto RTCP");
-	}
-      }
+        if (side[i][proto].sock != sock) {
+          msg.msg_name = (caddr_t ) &side[i][proto].sin;
+          msg.msg_namelen = sizeof(side[i][proto].sin);
+          msg.msg_accrights = 0;
+          msg.msg_accrightslen = 0;
+          if ((sendmsg(side[i][2].sock, &msg,0))!= 
+            iov[0].iov_len +iov[1].iov_len)
+            perror("sendmsg RTCP");
+          }
+       }
     }
-    else /* control messages */{
+    else if (((struct CtrlMsgHdr *)packet)->type == 1) /* vat ID messages */{
+      rtcp_t *rtcp_msg;
       struct sdes_msg *ctl_msg;
       rtcp_sdes_item_t *item;
       int len=0;
       int length;
       item=NULL;
 
-      /*total length of the packet = IP address+ site entry of vat+ 2 type+ 2 
-        length + 4 common header+ 4 ssrc */
-      length=strlen(inet_ntoa(sin_from.sin_addr))+
-	strlen(packet+sizeof(struct IDMsgHdr))+4+12;
-	length=((length/4)*4)+4;
-      ctl_msg=(struct sdes_msg *)malloc(length);
-      memset(ctl_msg,0,length);
+      /* total length of the packet = IP address+ site entry of vat+ 2 type+ 2 
+        length + 4 common header+ 4 ssrc + 8 empty RR */
+      length = strlen(inet_ntoa(sin_from.sin_addr)) +
+        strlen(packet+sizeof(struct CtrlMsgHdr)) + 12 + 8;
+      length = ((length/4)*4)+4;
+      rtcp_msg=(rtcp_t *)malloc(length);
+      memset(rtcp_msg,0,length);
 
+      /* init RR */
+      rtcp_msg->common.version=2;
+      rtcp_msg->common.p=0;
+      rtcp_msg->common.count=0;
+      rtcp_msg->common.pt=201;
+      rtcp_msg->r.rr.ssrc=sin_from.sin_addr.s_addr;
+      rtcp_msg->common.length=(8 >> 2) - 1;
+
+      ctl_msg=(struct sdes_msg *)&rtcp_msg->r.rr.rr[0];
       item=( rtcp_sdes_item_t *) ((char *)ctl_msg+8);
       /* init CNAME */
       item->type=1;
@@ -310,7 +345,7 @@ static Notify_value socket_handler(Notify_client client, int sock)
 
       item=(rtcp_sdes_item_t *)((char *)item +len);
       /* init NAME */
-      strcpy(item->data,packet+sizeof(struct IDMsgHdr)-4);
+      strcpy(item->data,packet+sizeof(struct CtrlMsgHdr));
       item->length=strlen(item->data);
       item->type=2;
       len+=item->length+2;
@@ -321,18 +356,18 @@ static Notify_value socket_handler(Notify_client client, int sock)
       ctl_msg->header.count=1;
       ctl_msg->header.pt=202;
       ctl_msg->sdes.src=sin_from.sin_addr.s_addr;
-      ctl_msg->header.length=length-4;
-			       
+      ctl_msg->header.length=((length-8) >> 2) - 1;
+
       for (i = 0; i < hostc; i++) {
-	if (side[i][proto].sock != sock) {
-	 if(sendto(side[i][proto].sock,(char *)ctl_msg,
-		   (ctl_msg->header.length+1)*4,
-		   0,(struct sockaddr *)&side[i][proto].sin,
-		   sizeof(side[i][proto].sin))<0)
-	    perror("sendto RTCP");
-	}
+        if (side[i][proto].sock != sock) {
+          if (sendto(side[i][2].sock,(char *)rtcp_msg,
+             ((rtcp_msg->common.length+1)+(ctl_msg->header.length+1))*4,
+             0, (struct sockaddr *)&side[i][proto].sin,
+             sizeof(side[i][proto].sin)) < 0)
+          perror("sendto RTCP");
+        }
       }
-      free(ctl_msg);
+      free(rtcp_msg);
     }/* control messages */
   }
   return NOTIFY_DONE;
@@ -362,7 +397,8 @@ int main(int argc, char *argv[])
   char loop = 0;  /* multicast loop */
   int reuse = 1;  /* reuse address */
   int i, j;
-  extern int hpt(char *h, struct sockaddr *sa, unsigned char *ttl);
+
+  extern struct in_addr host2ip(char *);
 
   while ((c = getopt(argc, argv, "d?h")) != EOF) {
     switch(c) {
@@ -384,17 +420,33 @@ int main(int argc, char *argv[])
 
   /* Parse host descriptions. */
   for (i = 0; i < argc-optind; i++) {
+    char *s;
+
     if (i >= MAX_HOST) break;
     host[i].ttl  = 16;
     host[i].name = argv[optind+i];
-    if (hpt(host[i].name, (struct sockaddr *)&host[i].sin, &host[i].ttl) < 0) {
+    host[i].sin.sin_family = AF_INET;
+    s = strchr(host[i].name, '/');
+    if (!s) {
       usage(argv[0]);
       exit(1);
     }
-    if (host[i].sin.sin_port & 1) {
-      fprintf(stderr, "%s: Port must be even.\n", argv[0]);
-      /* exit(1); */
+    else {
+      int port;
+
+      *s = '\0';
+      port = atoi(s+1);
+      if (port & 1) {
+        fprintf(stderr, "%s: Port must be even.\n", argv[0]);
+        /* exit(1); */
+      }
+      host[i].sin.sin_port = htons(port);
+      s = strchr(s+1, '/');
+      if (s) {
+        host[i].ttl = atoi(s+1);
+      }
     }
+    host[i].sin.sin_addr = host2ip(host[i].name);
     if (IN_CLASSD(host[i].sin.sin_addr.s_addr)) {
       host[i].mreq.imr_multiaddr        = host[i].sin.sin_addr;
       host[i].mreq.imr_interface.s_addr = htonl(INADDR_ANY);
@@ -404,7 +456,7 @@ int main(int argc, char *argv[])
 
   /* Create/bind sockets. */
   for (i = 0; i < hostc; i++) { /* hosts (unicast or multicast) */
-    for (j = 0; j < 2; j++) { /* ports (RTP, RTCP) */
+    for (j = 0; j < 3; j++) { /* receive ports (RTP, RTCP), send */
       side[i][j].sock = socket(PF_INET, SOCK_DGRAM, 0);
       if (side[i][j].sock < 0) {
         perror("socket");
@@ -413,19 +465,27 @@ int main(int argc, char *argv[])
       if (setsockopt(side[i][j].sock, SOL_SOCKET, SO_REUSEADDR, (char *)&reuse,
                    sizeof(reuse)) == -1)
         perror("setsockopt: reuseaddr");
-      side[i][j].sin = host[i].sin;
-      side[i][j].sin.sin_port = htons(ntohs(host[i].sin.sin_port) + j);
+      if (j < 2) {
+        side[i][j].sin = host[i].sin;
+        side[i][j].sin.sin_port = htons(ntohs(host[i].sin.sin_port) + j);
+      }
+      else {
+        side[i][j].sin.sin_addr.s_addr = INADDR_ANY;
+        side[i][j].sin.sin_port = 0;
+      }
 
       /* Bind to multicast address. */
       sin = side[i][j].sin;
       if (IN_CLASSD(host[i].sin.sin_addr.s_addr)) {
-	multi_sock[j]=side[i][j].sock;  /* save number of multicast socket */
-        if (setsockopt(side[i][j].sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
+        if (!multi_sock[j]) {
+          multi_sock[j]=side[i][j].sock;  /* save number of multicast socket */
+        }
+        if (j < 2 && setsockopt(side[i][j].sock, IPPROTO_IP, IP_ADD_MEMBERSHIP,
             (char *)&host[i].mreq, sizeof(host[i].mreq)) < 0) {
             perror("IP_ADD_MEMBERSHIP");
             exit(1);
         }
-        if (setsockopt(side[i][j].sock, IPPROTO_IP, IP_MULTICAST_TTL,
+        if (j==2 && setsockopt(side[i][j].sock, IPPROTO_IP, IP_MULTICAST_TTL,
             (char *)&host[i].ttl, sizeof(host[i].ttl)) < 0) {
           perror("IP_MULTICAST_TTL");
           exit(1);
@@ -441,7 +501,7 @@ again:
             exit(1);
           }
         }
-        if (setsockopt(side[i][j].sock, IPPROTO_IP, IP_MULTICAST_LOOP,
+        if (j==2 && setsockopt(side[i][j].sock, IPPROTO_IP, IP_MULTICAST_LOOP,
             (char *)&loop, sizeof(loop)) < 0) {
           perror("IP_MULTICAST_LOOP");
           exit(1);
@@ -455,8 +515,10 @@ again:
           exit(1);
         }
       }
-      notify_set_input_func((Notify_client)j, socket_handler, 
-        side[i][j].sock);
+      if (j < 2) {
+        notify_set_input_func((Notify_client)j, socket_handler, 
+          side[i][j].sock);
+      }
     } /* for j (protocols) */
   } /* for i (hosts) */
 
